# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from unittest import TestCase

from rhea import reader


class TestReader(TestCase):
    def test_reads_yaml_files(self):
        config = reader.read('tests/fixtures/yaml_file.yml')
        assert config == {'x': 10, 'y': 20, 'foo': 'bar', 'type': 'yaml'}

    def test_reads_json_files(self):
        config = reader.read('tests/fixtures/json_file.json')
        assert config == {'x': 1, 'y': 2, 'foo': 'bar', 'type': 'json'}

    def test_reads_config_map(self):
        config = reader.read([{'x': 'y'}, {1: 2}, {'x': 'override y'}])
        assert config == {'x': 'override y', 1: 2}

        config = reader.read([{'x': 'y'},
                              {1: 2},
                              {'x': 'override y'},
                              'tests/fixtures/yaml_file.yml',
                              'tests/fixtures/json_file.json'])
        assert config == {'x': 1, 'y': 2, 1: 2, 'foo': 'bar', 'type': 'json'}

    def test_reads_yaml_stream(self):
        stream = """---
        x: y
        1: 2
        """
        config = reader.read(stream)
        assert config == {'x': 'y', 1: 2}

    def test_reads_json_stream(self):
        stream = """---
        {x: y, 1: 2}
        """
        config = reader.read(stream)
        assert config is not None



import os

from unittest import TestCase

from rhea.exceptions import RheaError
from rhea.manager import Rhea
from rhea.specs import UriSpec


class TestRhea(TestCase):
    def setUp(self):
        os.environ['FOO_BAR_KEY'] = 'foo_bar'
        self.config = Rhea.read_configs(
            [os.environ,
             './tests/fixtures/configs/config_tests.json'])

    def test_get_from_os_env(self):
        assert self.config.get_string('FOO_BAR_KEY') == 'foo_bar'

    def test_reading_invalid_config_raises_error(self):
        with self.assertRaises(RheaError):
            Rhea.read_configs(
                ['./tests/fixtures/configs/invalid_config_tests.json'])

    def test_get_boolean(self):
        value = self.config.get_boolean('bool_key_1')
        self.assertEqual(value, True)

        value = self.config.get_boolean('bool_key_2')
        self.assertEqual(value, True)

        value = self.config.get_boolean('bool_key_3')
        self.assertEqual(value, False)

        value = self.config.get_boolean('bool_key_4')
        self.assertEqual(value, False)

        value = self.config.get_boolean('bool_list_key_1', is_list=True)
        self.assertEqual(value, [False, False, True, True, True, False])

        with self.assertRaises(RheaError):
            self.config.get_boolean('bool_error_key_1')

        with self.assertRaises(RheaError):
            self.config.get_boolean('bool_error_key_2')

        with self.assertRaises(RheaError):
            self.config.get_boolean('bool_error_key_3')

        with self.assertRaises(RheaError):
            self.config.get_boolean('bool_error_key_4')

        with self.assertRaises(RheaError):
            self.config.get_boolean('bool_error_key_5')

        with self.assertRaises(RheaError):
            self.config.get_boolean('bool_list_key_1')

        with self.assertRaises(RheaError):
            self.config.get_boolean('bool_list_error_key_2', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_boolean('bool_list_error_key_1', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_boolean('bool_key_1', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_boolean('bool_key_2', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_boolean('bool_non_existing_key')

        with self.assertRaises(RheaError):
            self.config.get_boolean('bool_non_existing_key', is_list=True)

        self.assertEqual(self.config.get_boolean('bool_non_existing_key', is_optional=True), None)
        self.assertEqual(self.config.get_boolean(
            'bool_non_existing_key', is_optional=True, default=True), True)

        self.assertEqual(self.config.get_boolean(
            'bool_non_existing_key', is_list=True, is_optional=True), None)
        self.assertEqual(self.config.get_boolean(
            'bool_non_existing_key', is_list=True, is_optional=True, default=[True, False]),
            [True, False])

    def test_get_int(self):
        value = self.config.get_int('int_key_1')
        self.assertEqual(value, 123)

        value = self.config.get_int('int_key_2')
        self.assertEqual(value, 123)

        value = self.config.get_int('int_list_key_1', is_list=True)
        self.assertEqual(value, [123, 124, 125, 125])

        with self.assertRaises(RheaError):
            self.config.get_int('int_error_key_1')

        with self.assertRaises(RheaError):
            self.config.get_int('int_error_key_2')

        with self.assertRaises(RheaError):
            self.config.get_int('int_error_key_3')

        with self.assertRaises(RheaError):
            self.config.get_int('int_list_key_1')

        with self.assertRaises(RheaError):
            self.config.get_int('int_list_error_key_1', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_int('int_list_error_key_2', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_int('int_list_error_key_3', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_int('int_key_1', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_int('int_key_2', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_int('int_non_existing_key')

        with self.assertRaises(RheaError):
            self.config.get_int('int_non_existing_key', is_list=True)

        self.assertEqual(self.config.get_int(
            'int_non_existing_key', is_optional=True), None)
        self.assertEqual(self.config.get_int(
            'int_non_existing_key', is_optional=True, default=34), 34)

        self.assertEqual(self.config.get_int(
            'int_non_existing_key', is_list=True, is_optional=True), None)
        self.assertEqual(self.config.get_int(
            'int_non_existing_key', is_list=True, is_optional=True, default=[34, 1]), [34, 1])

    def test_get_float(self):
        value = self.config.get_float('float_key_1')
        self.assertEqual(value, 1.23)

        value = self.config.get_float('float_key_2')
        self.assertEqual(value, 1.23)

        value = self.config.get_float('float_key_3')
        self.assertEqual(value, 123)

        value = self.config.get_float('float_list_key_1', is_list=True)
        self.assertEqual(value, [1.23, 13.3, 4.4, 555., 66.])

        with self.assertRaises(RheaError):
            self.config.get_float('float_error_key_1')

        with self.assertRaises(RheaError):
            self.config.get_float('float_error_key_2')

        with self.assertRaises(RheaError):
            self.config.get_float('float_error_key_3')

        with self.assertRaises(RheaError):
            self.config.get_float('float_error_key_4')

        with self.assertRaises(RheaError):
            self.config.get_float('float_list_key_1')

        with self.assertRaises(RheaError):
            self.config.get_float('float_list_error_key_1', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_float('float_list_error_key_2', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_float('float_list_error_key_3', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_float('float_key_1', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_float('float_key_2', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_float('float_non_existing_key')

        with self.assertRaises(RheaError):
            self.config.get_float('float_non_existing_key', is_list=True)

        self.assertEqual(self.config.get_float(
            'float_non_existing_key', is_optional=True), None)
        self.assertEqual(self.config.get_float(
            'float_non_existing_key', is_optional=True, default=3.4), 3.4)

        self.assertEqual(self.config.get_float(
            'float_non_existing_key', is_list=True, is_optional=True), None)
        self.assertEqual(self.config.get_float(
            'float_non_existing_key', is_list=True, is_optional=True, default=[3.4, 1.2]),
            [3.4, 1.2])

    def test_get_string(self):
        value = self.config.get_string('string_key_1')
        self.assertEqual(value, "123")

        value = self.config.get_string('string_key_2')
        self.assertEqual(value, "1.23")

        value = self.config.get_string('string_key_3')
        self.assertEqual(value, "foo")

        value = self.config.get_string('string_key_4')
        self.assertEqual(value, "")

        value = self.config.get_string('string_list_key_1', is_list=True)
        self.assertEqual(value, ["123", "1.23", "foo", ""])

        with self.assertRaises(RheaError):
            self.config.get_string('string_error_key_1')

        with self.assertRaises(RheaError):
            self.config.get_string('string_error_key_2')

        with self.assertRaises(RheaError):
            self.config.get_string('string_error_key_3')

        with self.assertRaises(RheaError):
            self.config.get_string('string_error_key_4')

        with self.assertRaises(RheaError):
            self.config.get_string('string_list_key_1')

        with self.assertRaises(RheaError):
            self.config.get_string('string_list_error_key_1', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_string('string_list_error_key_2', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_string('string_list_error_key_3', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_string('string_list_error_key_4', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_string('string_key_3', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_string('string_key_4', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_string('string_non_existing_key')

        with self.assertRaises(RheaError):
            self.config.get_string('string_non_existing_key', is_list=True)

        self.assertEqual(self.config.get_string(
            'string_non_existing_key', is_optional=True), None)
        self.assertEqual(self.config.get_string(
            'string_non_existing_key', is_optional=True, default='foo'), 'foo')

        self.assertEqual(self.config.get_string(
            'string_non_existing_key', is_list=True, is_optional=True), None)
        self.assertEqual(self.config.get_string(
            'string_non_existing_key', is_list=True, is_optional=True, default=['foo', 'bar']),
            ['foo', 'bar'])

    def test_get_dict(self):
        value = self.config.get_dict('dict_key_1')
        self.assertEqual(value, {"key1": "foo", "key2": 2, "key3": False, "key4": "1"})

        value = self.config.get_dict('dict_list_key_1', is_list=True)
        self.assertEqual(value, [
            {"key1": "foo", "key2": 2, "key3": False, "key4": "1"},
            {"key3": True, "key4": "2"},
            {"key1": False, "key2": "3"}
        ])

        with self.assertRaises(RheaError):
            self.config.get_dict('dict_error_key_1')

        with self.assertRaises(RheaError):
            self.config.get_dict('dict_error_key_2')

        with self.assertRaises(RheaError):
            self.config.get_dict('dict_error_key_3')

        with self.assertRaises(RheaError):
            self.config.get_dict('dict_error_key_4')

        with self.assertRaises(RheaError):
            self.config.get_dict('dict_list_key_1')

        with self.assertRaises(RheaError):
            self.config.get_dict('dict_list_error_key_1', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_dict('dict_list_error_key_2', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_dict('dict_list_error_key_3', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_dict('dict_list_error_key_4', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_dict('dict_key_1', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_dict('dict_non_existing_key')

        with self.assertRaises(RheaError):
            self.config.get_dict('dict_non_existing_key', is_list=True)

        self.assertEqual(self.config.get_dict('dict_non_existing_key', is_optional=True), None)
        self.assertEqual(self.config.get_dict(
            'dict_non_existing_key', is_optional=True, default={'foo': 'bar'}), {'foo': 'bar'})

        self.assertEqual(self.config.get_dict(
            'dict_non_existing_key', is_list=True, is_optional=True), None)
        self.assertEqual(self.config.get_dict(
            'dict_non_existing_key', is_list=True, is_optional=True, default=[
                {'foo': 'bar'}, {'foo': 'boo'}]),
            [{'foo': 'bar'}, {'foo': 'boo'}])

    def test_get_uri(self):
        value = self.config.get_uri('uri_key_1')
        self.assertEqual(value, UriSpec("user", "pass", "siteweb.ca"))

        value = self.config.get_uri('uri_key_2')
        self.assertEqual(value, UriSpec("user2", "pass", "localhost:8080"))

        value = self.config.get_uri('uri_key_3')
        self.assertEqual(value, UriSpec("user2", "pass", "https://quay.io"))

        value = self.config.get_uri('uri_list_key_1', is_list=True)
        self.assertEqual(value, [
            UriSpec("user", "pass", "siteweb.ca"),
            UriSpec("user2", "pass", "localhost:8080"),
            UriSpec("user2", "pass", "https://quay.io")
        ])

        with self.assertRaises(RheaError):
            self.config.get_uri('uri_error_key_1')

        with self.assertRaises(RheaError):
            self.config.get_uri('uri_error_key_2')

        with self.assertRaises(RheaError):
            self.config.get_uri('uri_error_key_3')

        with self.assertRaises(RheaError):
            self.config.get_uri('uri_error_key_4')

        with self.assertRaises(RheaError):
            self.config.get_uri('uri_list_key_1')

        with self.assertRaises(RheaError):
            self.config.get_uri('uri_list_error_key_1', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_uri('uri_list_error_key_2', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_uri('uri_list_error_key_3', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_uri('uri_list_error_key_4', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_uri('uri_key_1', is_list=True)

        with self.assertRaises(RheaError):
            self.config.get_uri('uri_non_existing_key')

        with self.assertRaises(RheaError):
            self.config.get_uri('uri_non_existing_key', is_list=True)

        self.assertEqual(self.config.get_uri('uri_non_existing_key', is_optional=True), None)
        self.assertEqual(
            self.config.get_uri('uri_non_existing_key',
                                is_optional=True,
                                default=UriSpec("user2", "pass", "localhost:8080")),
            UriSpec("user2", "pass", "localhost:8080"))

        self.assertEqual(
            self.config.get_uri('uri_non_existing_key', is_list=True, is_optional=True), None)
        self.assertEqual(
            self.config.get_uri('uri_non_existing_key',
                                is_list=True,
                                is_optional=True,
                                default=[UriSpec("user", "pass", "siteweb.ca"),
                                         UriSpec("user2", "pass", "localhost:8080")]),
            [UriSpec("user", "pass", "siteweb.ca"), UriSpec("user2", "pass", "localhost:8080")])

    def test_get_list(self):
        value = self.config.get_list('list_key_1')
        self.assertEqual(value, ['user:pass@siteweb.ca', "'pp'", '0.1', "'foo'"])

        value = self.config.get_list('list_key_2')
        self.assertEqual(value, ['user1', 'user2', 'user3', 'user4', 'user5'])

        value = self.config.get_list('list_key_3')
        self.assertEqual(value, [False])

        value = self.config.get_list('list_key_4')
        self.assertEqual(value, ['foo'])

        value = self.config.get_list('list_key_5')
        self.assertEqual(value, [])

        with self.assertRaises(RheaError):
            self.config.get_list('list_error_key_1')

        with self.assertRaises(RheaError):
            self.config.get_list('list_error_key_2')

        with self.assertRaises(RheaError):
            self.config.get_list('list_error_key_3')

        with self.assertRaises(RheaError):
            self.config.get_list('list_error_key_4')

        with self.assertRaises(RheaError):
            self.config.get_list('list_non_existing_key')

        self.assertEqual(self.config.get_list(
            'list_non_existing_key', is_optional=True), None)
        self.assertEqual(self.config.get_list(
            'list_non_existing_key', is_optional=True, default=['foo']), ['foo'])

    def test_get_dict_of_dicts(self):
        value = self.config.get_dict_of_dicts('dict_dicts_key_1')
        self.assertEqual(value, {'data1': {'mountPath': '/data/21', 'existingClaim': 'data-1-pvc'}})

        value = self.config.get_dict_of_dicts('dict_dicts_key_2')
        self.assertEqual(value, {
            'outputs1': {'mountPath': '/output/2', 'existingClaim': 'outputs-1-pvc'},
            'outputs2': {'mountPath': '/output/2', 'existingClaim': 'output-2-pvc'}})

        value = self.config.get_dict_of_dicts('dict_dicts_key_3')
        self.assertEqual(value, None)

        with self.assertRaises(RheaError):
            self.config.get_dict_of_dicts('dict_dicts_error_key_1')

        with self.assertRaises(RheaError):
            self.config.get_dict_of_dicts('dict_dicts_error_key_2')

        with self.assertRaises(RheaError):
            self.config.get_dict_of_dicts('dict_dicts_error_key_3')

        with self.assertRaises(RheaError):
            self.config.get_dict_of_dicts('dict_dicts_error_key_4')

        with self.assertRaises(RheaError):
            self.config.get_dict_of_dicts('dict_dicts_error_key_4')

        with self.assertRaises(RheaError):
            self.config.get_dict_of_dicts('dict_dicts_error_key_5')

        with self.assertRaises(RheaError):
            self.config.get_dict_of_dicts('dict_dicts_error_key_6')

        with self.assertRaises(RheaError):
            self.config.get_dict_of_dicts('dict_dicts_error_key_7')

        with self.assertRaises(RheaError):
            self.config.get_dict_of_dicts('dict_dicts_non_existing_key')

        self.assertEqual(self.config.get_dict_of_dicts(
            'dict_dicts_non_existing_key', is_optional=True), None)
        self.assertEqual(self.config.get_dict_of_dicts(
            'dict_dicts_non_existing_key', is_optional=True, default={}), {})



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import six

from collections import Mapping


def deep_update(config, override_config):
    for k, v in six.iteritems(override_config):
        if isinstance(v, Mapping):
            k_config = config.get(k, {})
            if isinstance(k_config, Mapping):
                v_config = deep_update(k_config, v)
                config[k] = v_config
            else:
                config[k] = v
        else:
            config[k] = override_config[k]
    return config


def to_list(value):
    try:
        import numpy as np

        if isinstance(value, np.ndarray):
            return list(value)

    except ImportError:
        pass

    if isinstance(value, (list, tuple)):
        return list(value)
    return [value]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import json
import os
import six
import yaml

from collections import Mapping
from yaml.scanner import ScannerError  # noqa

from rhea.exceptions import RheaError
from rhea.utils import deep_update, to_list


def read(config_values):
    """Reads an ordered list of configuration values and deep merge the values in reverse order."""
    if not config_values:
        raise RheaError('Cannot read config_value: `{}`'.format(config_values))

    config_values = to_list(config_values)

    config = {}
    for config_value in config_values:
        if not isinstance(config_value, (Mapping, six.string_types)):
            raise RheaError(
                "Expects Mapping, string, or list of Mapping/string instances, "
                "received {} instead".format(type(config_value)))

        if isinstance(config_value, Mapping):
            config_results = config_value
        elif os.path.isfile(config_value):
            config_results = _read_from_file(config_value)
        else:
            # try reading a stream of yaml or json
            try:
                config_results = _read_from_stream(config_value)
            except ScannerError:
                raise RheaError(
                    'Received non valid yaml stream: `{}`'.format(config_value))

        if config_results and isinstance(config_results, Mapping):
            config = deep_update(config, config_results)
        else:
            raise RheaError('Cannot read config_value: `{}`'.format(config_value))

    return config


def _read_from_stream(stream):
    results = _read_from_yml(stream, is_stream=True)
    if not results:
        results = _read_from_json(stream, is_stream=True)
    return results


def _read_from_file(f_path):
    _, ext = os.path.splitext(f_path)
    if ext in ('.yml', '.yaml'):
        return _read_from_yml(f_path)
    elif ext == '.json':
        return _read_from_json(f_path)
    raise RheaError(
        "Expects a file with extension: `.yml`, `.yaml`, or `json`, "
        "received instead `{}`".format(ext))


def _read_from_yml(f_path, is_stream=False):
    if is_stream:
        return yaml.safe_load(f_path)
    with open(f_path) as f:
        return yaml.safe_load(f)


def _read_from_json(f_path, is_stream=False):
    if is_stream:
        try:
            return json.loads(f_path)
        except ValueError as e:
            raise RheaError(e)
    try:
        return json.loads(open(f_path).read())
    except ValueError as e:
        raise RheaError(e)
